/*
 * keytree_engine.h
 */

#include "seal.h"

/* The keytree structure */
typedef struct keytree_tree_node_s * KEYTREE_tree_node_p_t;
typedef struct keytree_mem_node_s * KEYTREE_mem_node_p_t;
typedef struct keytree_control_s * KEYTREE_control_p_t;

typedef struct keytree_tree_node_s {
	KEYTREE_tree_node_p_t left;
	KEYTREE_tree_node_p_t right;
	KEYTREE_tree_node_p_t parent;
	KEYTREE_mem_node_p_t mem_node;
	int tree_node_id;
	int renewed;
} KEYTREE_tree_node_t;

typedef struct keytree_mem_node_s {
	KEYTREE_tree_node_p_t tree_node;
	KEYTREE_mem_node_p_t prev;
	KEYTREE_mem_node_p_t next;
	char mem_guid[SEAL_GUID_LEN];
	int sponsor;
} KEYTREE_mem_node_t;

typedef struct keytree_control_s {
	KEYTREE_tree_node_p_t root;				// point to the root key node 
	KEYTREE_mem_node_p_t m_list_head, m_list_tail;	// storing the members
	KEYTREE_control_p_t subtree;			// a subtree
	pthread_mutex_t lock;
} KEYTREE_control_t;

/* Keytree INITIALIZATION */
KEYTREE_control_p_t KEYTREE_init();
void KEYTREE_create_from_size(KEYTREE_control_p_t tree, char* const* members, 
		int size);
void KEYTREE_create_from_list(KEYTREE_control_p_t tree, char* const* members, 
		const int* tree_node_ids, int size);

/* Keytree REBUILD */
void KEYTREE_rebuild(KEYTREE_control_p_t tree, char* const* joins, 
		int join_size, char* const* leaves, int leave_size);

/* Keytree: BATCH */
void KEYTREE_batch(KEYTREE_control_p_t tree, char* const* joins, int join_size, 
		char* const* leaves, int leave_size);

/* Keytree: QBATCH */
void KEYTREE_queue_subtree(KEYTREE_control_p_t subtree, char* const* joins, 
		int join_size, char* const* leaves, int leave_size);
void KEYTREE_queue_merge(KEYTREE_control_p_t tree, char* const* joins,
		int join_size, char* const* leaves, int leave_size);

/* Keytree: sponsors' coordination algorithm */
int KEYTREE_sponsor_coordinate(KEYTREE_control_p_t tree, char const* guid, 
		int* bcast_list);

/* Keytree: helper functions */
void KEYTREE_unmark(KEYTREE_control_p_t); 
int KEYTREE_get_num_of_members(KEYTREE_control_p_t);
int KEYTREE_get_members_tree_node_ids(KEYTREE_control_p_t t, char** members, 
		int* tree_node_ids);
int KEYTREE_get_keypath_len(KEYTREE_control_p_t t, char const* guid);
int KEYTREE_get_keypath(KEYTREE_control_p_t t, char const* guid, int* keypath);
int KEYTREE_get_renewed(KEYTREE_control_p_t t, char const* guid, int* renewed);
int KEYTREE_is_empty(KEYTREE_control_p_t tree);
int KEYTREE_is_guid_exist(KEYTREE_control_p_t tree, char const* guid);

/* Keytree: debug functions */
void KEYTREE_print(KEYTREE_tree_node_p_t);

/* Keytree: destroy */
void KEYTREE_destroy(KEYTREE_control_p_t);

