/*
 * message queue.h
 */

/*** structure declaration ***/
typedef struct msg_queue_node_s * MSGQUEUE_node_p_t;
typedef struct msg_queue_node_s {
	int rekey_seq;
	char msg_sender[SEAL_GUID_LEN];	// sender assumed in GUID format
	void* msg;						// message pointer
	int msg_size;					// message size
	MSGQUEUE_node_p_t prev;		
	MSGQUEUE_node_p_t next;
} MSGQUEUE_node_t;

typedef struct msg_queue_s * MSGQUEUE_control_p_t;
typedef struct msg_queue_s {
	MSGQUEUE_node_p_t head;
	MSGQUEUE_node_p_t tail;
	int queue_size;				// queue size
	pthread_cond_t cond;		// condition variables
	pthread_mutex_t lock;		// lock
} MSGQUEUE_control_t;

/*** function prototypes ***/
MSGQUEUE_control_p_t MSGQUEUE_init();

/*** queue management ***/
void MSGQUEUE_enq(MSGQUEUE_control_p_t msg_queue, int rekey_seq, 
		char const* msg_sender, void const* msg, int msg_size);
void MSGQUEUE_deq(MSGQUEUE_control_p_t msg_queue, char* msg_sender, void* msg);
int MSGQUEUE_get_size(MSGQUEUE_control_p_t msg_queue);
int MSGQUEUE_get_head_rekey_seq(MSGQUEUE_control_p_t msg_queue);
int MSGQUEUE_get_head_msg_size(MSGQUEUE_control_p_t msg_queue);

/*** destroy ***/
void MSGQUEUE_destroy(MSGQUEUE_control_p_t msg_queue);

