/*
 * packet.h 
 */

#include "seal.h"

/***** constants *****/
#define PACKET_REKEY			0x01
#define PACKET_BKEY				0x02
#define PACKET_MSG				0x03

/***** structure declaration *****/
// define a rekey packet
typedef struct packet_rekey_s * PACKET_rekey_p_t;
typedef struct packet_rekey_s {
	char rekey_seq[4];			// sequence number of rekeying
	char rekey_mode;			// tell which rekeying algorithm
	char no_mnode[4];			// no of member nodes in keytree
	char no_join[4];			// no of joins
	char no_leave[4];			// no of leaves
	
	char data[SEAL_MAX_PKT_SIZE];	// variable size
	// 1) array of nodes of the keytree
	//    node in (guid,tree_node_id), 36 bytes each
	// 2) array of join members' guids (32 bytes each),
	// 3) array of leave members' guids (32 bytes each)
	// 4) signature (if signature flag is defined in Makefile)

} PACKET_rekey_t;

// define the pub key packet 
typedef struct packet_bli_key_s * PACKET_bli_key_p_t;
typedef struct packet_bli_key_s {
	char rekey_seq[4];				// sequence number of rekeying
	char rekey_mode;				// tell which rekeying algorithm
	char no_bli_key[4];				// no of pub keys 

	char data[SEAL_MAX_PKT_SIZE];	// variable size
	// 1) array of blinded keys
	// element in (tree node id, pub key), (4 + SESSKEY_LEN) bytes each
	// 2) signature (if signature flag is defined in Makefile)
	
} PACKET_bli_key_t;

// define a message packet
typedef struct packet_msg_s * PACKET_msg_p_t;
typedef struct packet_msg_s {
	char rekey_seq[4];				// 0 means unencrypted
	
	char data[SEAL_MAX_PKT_SIZE];	// variable size 
	// 1) message length (NOT include signature)
	// 2) messages	
	// 3) signature (if signature flag is defined in Makefile)
	// *) All parts will be encrypted
	
} PACKET_msg_t;

// define the packet control structure
typedef struct packet_control_s * PACKET_control_p_t;
typedef struct packet_control_s {
	int mbox;
	char group[SEAL_GROUP_LEN];
	pthread_mutex_t lock;
} PACKET_control_t;

/***** function prototypes *****/

/*** packet engine ***/
PACKET_control_p_t PACKET_init(int mbox, char const* group);
int PACKET_send(PACKET_control_p_t packet_engine, int packet_type,
		int data_len, void const* packet);
void PACKET_destroy(PACKET_control_p_t packet_engine);

/*** packet create methods ***/
int PACKET_create_rekey(int seq, char mode, int no_mnode, int no_join,
		int no_leave, char* const* mnode_guids, int* tree_node_ids,
		char* const* join_guids, char* const* leave_guids,
		PACKET_rekey_p_t ret_pkt);
int PACKET_create_bli_key(int seq, char rekey_mode, int no_bli_key,
		int const* tree_node_ids, char* const* bcast_keys,
		PACKET_bli_key_p_t ret_pkt);
int PACKET_create_msg(int seq, char const* msg, int pkt_len,
		PACKET_msg_p_t ret_pkt);


