/*
 * ui_engine.h
 */

#include "chatter.h"

/***** constants *****/
#define UI_MAX_MSG_SIZE		3000		// maximum message size to be sent
#define UI_MAX_PRINT_LEN	100			// maximum printf length
#define UI_MAX_CHAR			100			// max characters in each arg 
										//	(in UI_textmode_split_arg())
#define UI_MAX_ARG			40			// max no. of arguments
										//	(in UI_textmode_split_arg())

#define UI_GROUP_IN			0x00		// inside group (in textmode)
#define UI_GROUP_OUT		0x01		// outside group (in textmode)

/*** error codes ***/
#define UI_ERR_GROUP_NOT_VALID			0xF0	// not valid group name
#define UI_ERR_GROUP_JOINED				0xF1	// already in the group
#define UI_ERR_GROUP_LEFT				0xF2	// not in any group
#define UI_ERR_GROUP_NO_MEMBER			0xF3	// zero member in the group
#define UI_ERR_MSG_NOT_ENCRYPTED		0xF4	// message not encrypted
#define UI_ERR_MEMBER_NOT_AUTHORIZED	0xF5	// member not valid

/***** structure declarations *****/
typedef struct ui_control_s * UI_control_p_t;
typedef struct ui_control_s {

#ifdef GUI_ENABLED				// **** gui mode ****
	// widgets in the main window
	GtkWidget *main_window;
	GtkWidget *main_vbox;
	GtkWidget *main_table;
	GtkWidget *main_button_hbox;

	GtkWidget *input_frame;
	GtkWidget *input_hbox;
	GtkWidget *input_label;
	GtkWidget *input_text_entry;
	GtkWidget *input_button;

	GtkWidget *msg_frame;
	GtkWidget *msg_vbox;
	GtkWidget *msg_scrolled_window;	
	GtkWidget *msg_clist;
	GtkWidget *msg_clear_button;

	GtkWidget *member_frame;
	GtkWidget *member_table;
	GtkWidget *member_scrolled_window;	
	GtkWidget *member_clist;
	GtkWidget *member_group_label;
	GtkWidget *member_group_entry;
	GtkWidget *member_join_button;
	GtkWidget *member_leave_button;

	GtkWidget *status_frame;
	GtkWidget *status_scrolled_window;	
	GtkWidget *status_clist;
	
	GtkWidget *button_exit;

#ifdef SIGNATURE		// defined signature flag
	GtkWidget* passwd_window;
	GtkWidget* passwd_label;
	GtkWidget* passwd_entry;
	GtkWidget* passwd_button_ok;
	GtkWidget* passwd_button_cancel;
#endif					// end: defined signature flag

	pthread_t poll_member_thread;	// thread for polling members in GUI mode

#endif							// **** end: gui mode ****

	// both gui mode and textmode
	int state;						// inside group / outside group
	pthread_t msg_recv_thread;		// thread for handling received messages

} UI_control_t;

/***** function prototypes *****/

/*** For both GUI and Textmode ***/
UI_control_p_t UI_init(int argc, char** argv);
void UI_start(UI_control_p_t ui_engine);
void UI_print_members(UI_control_p_t ui_engine, char* const* members,
		int no_member);
void UI_print_msg(UI_control_p_t ui_engine, char const* sender, 
		char const* msg);
void UI_print_error(UI_control_p_t ui_engine, int err_code, char* arg);
void* UI_receive_handler(UI_control_p_t ui_engine);

#ifdef GUI_ENABLED
/*** GUI ***/
/*** initialization functions ***/
void UI_init_gui_main(UI_control_p_t ui_engine);
#ifdef SIGNATURE
void UI_init_gui_passwd(UI_control_p_t ui_engine);
#endif
/*** callback functions ***/
#ifdef SIGNATURE
void UI_gui_passwd_check(GtkWidget* widget, UI_control_p_t ui_engine);
#endif
void UI_gui_input_send(GtkWidget* widget, UI_control_p_t ui_engine);
void UI_gui_msg_clear(GtkWidget* widget, UI_control_p_t ui_engine);
void UI_gui_join(GtkWidget* widget, UI_control_p_t ui_engine);
void UI_gui_leave(GtkWidget* widget, UI_control_p_t ui_engine);
void UI_gui_exit();
void* UI_gui_poll_members(UI_control_p_t ui_engine);
/*** end: GUI ***/
#else
/*** Textmode functions ***/
#ifdef SIGNATURE
void UI_textmode_passwd_check();
#endif
int UI_textmode_split_arg(char const* input, char** arg);
void UI_textmode_join(UI_control_p_t ui_engine, int no_arg, char* const* arg);
void UI_textmode_leave(UI_control_p_t ui_engine, int no_arg, char* const* arg);
void UI_textmode_member(UI_control_p_t ui_engine, int no_arg, char* const* arg);
void UI_textmode_msg(UI_control_p_t ui_engine, char const* text);
void UI_textmode_exit(int no_arg, char* const* arg);
/*** end: Textmode functions ***/
#endif
